#include "stdafx.h"
#include "PJClock.h"
#include "resource.h"
#include "MainFrm.h"

constexpr UINT MOUSE_MOVE = 0xF012;


#ifdef _DEBUG
#define new DEBUG_NEW
#endif


#pragma warning(suppress: 26409 26433 26440 26477)
IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

#pragma warning(suppress: 26433 26440)
BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd) //NOLINT(modernize-avoid-c-arrays)
  ON_WM_CONTEXTMENU()
  ON_COMMAND(ID_FILE_TRANSPARENT, &CMainFrame::OnFileTransparent)
  ON_UPDATE_COMMAND_UI(ID_FILE_TRANSPARENT, &CMainFrame::OnUpdateFileTransparent)
  ON_COMMAND(ID_FILE_ALWAYSONTOP, &CMainFrame::OnFileAlwaysontop)
  ON_UPDATE_COMMAND_UI(ID_FILE_ALWAYSONTOP, &CMainFrame::OnUpdateFileAlwaysontop)
  ON_COMMAND(ID_FILE_CAPTION, &CMainFrame::OnFileCaption)
  ON_UPDATE_COMMAND_UI(ID_FILE_CAPTION, &CMainFrame::OnUpdateFileCaption)
  ON_WM_SIZE()
  ON_WM_RBUTTONDOWN()
  ON_WM_LBUTTONDBLCLK()
  ON_WM_LBUTTONDOWN()
  ON_WM_CREATE()
END_MESSAGE_MAP()

CMainFrame::CMainFrame() noexcept : m_bTopMostWindow{false}
{
}

#pragma warning(suppress: 26434)
int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
  //Let the base class do its thing
  if (__super::OnCreate(lpCreateStruct) == -1)
    return -1;

  //Load up the main menu
  if (!m_MainMenu.LoadMenu(IDR_MAINFRAME))
    return -1;

  return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
  //Let the parent class do its thing
  const BOOL bSuccess{__super::PreCreateWindow(cs)};

  //Remove the client edge
  cs.dwExStyle &= ~WS_EX_CLIENTEDGE;
  return bSuccess;
}

void CMainFrame::OnFileTransparent()
{
  const bool bLayered{(GetExStyle() & WS_EX_LAYERED) != 0};
  if (!bLayered)
  {
    //Add the transparent style
    ModifyStyleEx(0, WS_EX_LAYERED);

    //Make the Window transparent used the button face color
    SetLayeredWindowAttributes(GetSysColor(COLOR_BTNFACE), 0, LWA_COLORKEY);
  }
  else
  {
    //Remove the layered style
    ModifyStyleEx(WS_EX_LAYERED, 0);

    //Ask the window and its children to repaint
    ::RedrawWindow(GetSafeHwnd(), nullptr, nullptr, RDW_ERASE | RDW_INVALIDATE | RDW_FRAME | RDW_ALLCHILDREN); //NOLINT(clang-analyzer-core.CallAndMessage)
  }
}

#pragma warning(suppress: 26429)
void CMainFrame::OnUpdateFileTransparent(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486)
  pCmdUI->SetCheck(GetExStyle() & WS_EX_LAYERED);
}

void CMainFrame::OnFileAlwaysontop()
{
  m_bTopMostWindow = !m_bTopMostWindow;

  if (m_bTopMostWindow)
    SetWindowPos(&CWnd::wndTopMost, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW);
  else
  {
    SetRedraw(FALSE);
    SetWindowPos(&CWnd::wndBottom, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
    BringWindowToTop();
    SetRedraw(TRUE);
  }
}

#pragma warning(suppress: 26429)
void CMainFrame::OnUpdateFileAlwaysontop(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486)
  pCmdUI->SetCheck(m_bTopMostWindow);
}

void CMainFrame::OnFileCaption()
{
  const bool bCaption{(GetStyle() & WS_CAPTION) != 0};
  if (!bCaption)
  {
    //Add the caption and thick frame styles
    ModifyStyle(0, WS_CAPTION | WS_THICKFRAME);

    //Hook up the menu again
    SetMenu(&m_MainMenu);
  }
  else
  {
    //Remove the caption and thick frame styles
    ModifyStyle(WS_CAPTION | WS_THICKFRAME, 0);

    //Also remove the menu
    SetMenu(nullptr);
  }

  //Ask the window and its children to repaint
  ::RedrawWindow(GetSafeHwnd(), nullptr, nullptr, RDW_ERASE | RDW_INVALIDATE | RDW_FRAME | RDW_ALLCHILDREN);
}

#pragma warning(suppress: 26429)
void CMainFrame::OnUpdateFileCaption(CCmdUI* pCmdUI)
{
#pragma warning(suppress: 26486)
  pCmdUI->SetCheck(GetStyle() & WS_CAPTION);
}

#pragma warning(suppress: 26434)
void CMainFrame::OnSize(UINT nType, int cx, int cy)
{
  //Let the parent class do its thing
  __super::OnSize(nType, cx, cy);

  //Resize the clock control
  CRect rClient;
  GetClientRect(&rClient);
  m_ctrlClock.MoveWindow(rClient);
}

BOOL CMainFrame::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext)
{
  //Create the clock control
  CRect rClient;
  GetClientRect(&rClient);
  if (!m_ctrlClock.Create(_T(""), WS_CHILD | WS_VISIBLE | SS_SIMPLE, rClient, this))
    return 0;

  //Let the parent class do its thing
  return __super::OnCreateClient(lpcs, pContext);
}

#pragma warning(suppress: 26434)
void CMainFrame::OnLButtonDown(UINT /*nFlags*/, CPoint /*point*/)
{
  //Fake a Window drag
  SendMessage(WM_LBUTTONUP);
  SendMessage(WM_SYSCOMMAND, MOUSE_MOVE);
}

#pragma warning(suppress: 26434)
void CMainFrame::OnContextMenu(CWnd*, CPoint point)
{
  if ((point.x == -1) && (point.y == -1))
  {
    //keystroke invocation
    CRect rect;
    GetClientRect(rect);
    ClientToScreen(rect);

    point = rect.TopLeft();
    point.Offset(5, 5);
  }

  //Load up the menu
  CMenu menu;
  if (!menu.LoadMenu(IDR_POPUP_MAINFRAME))
    return;

  //Show the menu
  CMenu* pPopup{menu.GetSubMenu(0)};
#pragma warning(suppress: 26496)
  AFXASSUME(pPopup != nullptr);
  pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,	this);
}

#pragma warning(suppress: 26434)
void CMainFrame::OnLButtonDblClk(UINT /*nFlags*/, CPoint /*point*/) 
{
  OnFileCaption();
}
